(document:surround "/std/frame")
(document:insert "/std/functions")

;;;
(define rlist (make-cell `()))

(define (rlist-add name label)
  (cell-set! rlist (cons (cons name (radio label)) (cell-ref rlist))) )

(define (rlist-set name)
  (for-each (lambda (p) (if (string=? (car p) name) ((cdr p) state #t)))
       (cell-ref rlist)))

(define (rlist-get)
  (any (lambda (p) (if ((cdr p) state) (car p) #f ))
       (cell-ref rlist)))

(define (rlist-widgets)
  (reverse (map cdr (cell-ref rlist))))
;;;


;;;
(define cblist (make-cell `()))

(define (cblist-add name label st)
  (cell-set! cblist (cons (cons name (checkbox label state st)) (cell-ref cblist))) )

(define (cblist-get)
  (apply string-append
    (map (lambda (p) (string-append (if ((cdr p) state) "+" "-") (car p) ";"))
         (cell-ref cblist))))

(define (cblist-widgets)
  (reverse (map cdr (cell-ref cblist))))
;;;


(define (skip-path name)
  (string-drop name (+ 1 (or (string-index-right name #\/) -1))))

(woo-catch/message (thunk
  (map (lambda (data) (rlist-add
          (woo-get-option data 'name "")
          (woo-get-option data 'label "") ))
       (woo-list "/lilo/bootloader_places") )
  (let ((data (woo-read-first "/lilo/dev") ))
    (rlist-set (woo-get-option data 'current "")) )))

(woo-catch/message (thunk
  (map (lambda (data) (cblist-add
          (woo-get-option data 'name "")
          (woo-get-option data 'long_label "")
          (woo-get-option data 'enabled #f) ))
       (woo-list "/lilo/others") )))

(define (round-timeout t)
  (let ((t (string->number t)))
   (cond
	 ((not (number? t)) "")
	 ((< t 10) "0")
	 ((< t 50) "10")
	 ((< t 100) "50")
	 ((< t 200) "100")
	 ((< t 600) "200")
	 (else "600"))))

(define (read-data)
  (woo-catch/message (thunk 
  (let* ((data  (woo-read-first "lilo/globals"))
         (t     (woo-get-option data 'timeout ""))
         (sects (woo-list "lilo/sections"))
         (defs  (find
                  (lambda(x)
                    (woo-get-option x 'default #f))
                 sects))
         (defname (woo-get-option defs 'name ""))
        )

    (timeout value (round-timeout t))

    (def-section value defname)
  ))))

(define (write-data)
  (woo-catch/message (thunk
    (woo-write "/lilo/globals" 'target "boot" 'devname (rlist-get))
    (woo-write "/lilo/simple_sections" 'sections (cblist-get))
    (woo-write "/lilo/globals" 'target "timeout" 'timeout (timeout value))
    (woo-write "/lilo/globals" 'target "default" 'default (def-section value))
)))

(define (write-n-commit)
  (woo-catch/message (thunk
    (woo-write "/lilo/globals" 'target "boot" 'devname (rlist-get))
    (woo-write "/lilo/simple_sections" 'sections (cblist-get))
    (woo-write "/lilo/globals" 'target "timeout" 'timeout (timeout value))
    (woo-write "/lilo/globals" 'target "default" 'default (def-section value))
    (woo-write "/lilo/commit" 'confonly #f))))


;(and (global 'frame:next)
;  (label (_ "Boot loader is used to boot your operating system and it can be configured here. If this is a sole operating system on this machine you should install the boot loader on hard drive. Otherwise, please consult the documentation for available options.") text-wrap #t))

(gridbox columns "50;0;50"
  (spacer)
  (apply groupbox (_ "Install boot loader on:") (rlist-widgets))
  (spacer)

  (spacer)
  (if (not (= 0 (length (cell-ref cblist))))
    (apply groupbox (_ "Load other OS:") (cblist-widgets))
    (spacer))
  (spacer)

  (spacer)
  (gridbox columns "0;100"

    (label text "" colspan 2)

    (label text (_ "Load by default:")    align 'right)
      (document:id def-section (combobox enumref "lilo/sections"))

    (label text (_ "Interface timeout:") align 'right)
      (document:id timeout (combobox enumref "lilo/std_timeouts"))

    (label text "" colspan 2)
  )
  (spacer)

  (spacer)
  (gridbox columns "50;50" align "top"
    (if (global 'frame:next)
      (spacer)
      (button text (_ "Apply") (when clicked (write-n-commit))))
    (button text (_ "Expert mode")
      (when clicked
        (write-data)
        (frame:replace "/lilo/expert"))))
  (spacer)


)

(and (global 'frame:next)
  (frame:on-next (thunk (or (write-n-commit) 'cancel))))

(document:root (when loaded
  (read-data)
  (and (global 'frame:next) (frame:back-activity #f))))

